#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <errno.h>

#include "fd_image.h"
#include "hd_image.h"

void usage(const char *msg)
{
	printf("jv1hack: %s\n", msg);
	printf("\n");
	printf("JV1 hack\n");
	printf("Converts input disk image to JV1 format\n");
	printf("Doesn't do error checking\n");
	printf("\n");
	printf("usage: jv1hack [-v] diskimage\n");
	printf("\t-v\tPrint progress information.\n");
	exit(1);
}

int verbose;
int	extract;

int main(int argc, char *argv[])
{
	DiskImageMetadata meta;
//QQQQ	DriveConfiguration drive_configuration[1] = { 300,	500*1024, DISK_FM, 1, 35, 10 };
	DriveConfiguration drive_configuration[1] = { 300,	500*1024, DISK_MFM, 1, 35, 18 };
	Notifier report;

	verbose = 0;
	extract = 0;
	const char* dump_file = NULL;

	for (; argc >= 2 && argv[1][0] == '-'; argc--, argv++) {
		if (strcmp(argv[1], "-v") == 0)
			verbose++;
		else if (strcmp(argv[1], "-e") == 0 || strcmp(argv[1], "-x") == 0)
			extract = 1;
		else if (strcmp(argv[1], "-dd") == 0) {
			if (argc < 3)
				usage("no file argument to -dd");
			dump_file = argv[2];
			argv++;
			argc--;
		}
		else
			usage("unknown argument");
	}

	if (argc < 2) {
		usage("no floppy image files given");
	}
	if (argc > 3) {
		usage("too many floppy images");
	}

	FILE* tar_out = NULL;

	const char* error = NULL;

	const char* path = argv[1];
	DiskDrive* dsk = LoadDisk(path, NULL, 0, error, drive_configuration[0]);
	if (error != NULL) { 
		fprintf(stderr, "jv1hack: error loading %s: %s\n", path, error);
		return -1;
	}

#if 0
	#define EXT "dmk"
	DMKDiskDrive *dest = new DMKDiskDrive(drive_configuration[0]);
#endif
#if 1
	#define EXT "jv3"
	JV3DiskDrive *dest = new JV3DiskDrive(drive_configuration[0]);
#endif
#if 0
	#define EXT "imd"
	IMDDiskDrive *dest = new IMDDiskDrive(drive_configuration[0]);
#endif
#if 0 // QQQ
	#define EXT "jv1"
	JV1DiskDrive *dest = new JV1DiskDrive(drive_configuration[0]);
#endif
{	
		const DriveConfiguration *dc = dsk->GetConfig();
		int num_sides = dsk->GetNumSides();
		int num_tracks = dc->tracks;
		if (verbose)
			printf("%s: sides %d tracks %d sectors %d\n", path, num_sides, num_tracks, dc->sectors_per_track);
	
		// iterate
		int z = 0;
		for (int track = 0; track < num_tracks; track++) {
			for (int side = 0; side < num_sides; side++) {

				TrackInfo trk;
				dsk->GetTrackInfo(trk, side, track);
				if (verbose) {
					if (num_sides < 2)
						printf("trk %02d %02d sectors\n", track, trk.sectors); 
					else
						printf("trk %02d side %d %02d sectors\n", track, side, trk.sectors); 
				}
				DMKTrack raw;
				dsk->GetDMKTrack(&raw, side, track);

				dest->PutDMKTrack(&raw, side, track);
			}
		}
		delete dsk;
	}

	char fname[256];
	sprintf(fname, "out.%s", EXT);
	FILE* f = fopen(fname, "wb");
	const char* err = dest->Save(f);
	fclose(f);

printf("saved to %s\n", fname);
if (err != NULL) {
	printf("  but with an error: '%s'\n", err);
}
	return 0;
}

// Interesting: out.imd "failed" for -m3 but "out.jv3" worked for -m1 and -m3
// Of course: can't "dual boot" an IMD.
//
// "out.jv1" is empty--because of the two-sector track.
// - at least JV1 admitted a failure
// - IMD did not.
// Can make JV1 load for Model I if I pad missing sectors
// and add to the detection rules.
/*
Thinking about conversion:

DMK -> block can use GetDMKTrack(class DMKTrack *trk, int side, int track);
- and then code from BlockDrive::NotifyWriteTrack() to write back

block -> dmk can use BlockDrive::GetDMKTrack(class DMKTrack *trk, int side, int track)
And then, presumably, some code to write it back to the DMK

block -> block can more-or-less copy the tracks data literally.

I'd certainly like to try and copy sledge

Well, a quick hack did it.
*/

void ods(char*){}

